/*----------------------------------------------------------------------------
 * Name:    encoder.c
 * Purpose: Mehanical encoder function
 * Note(s):
 *----------------------------------------------------------------------------*/
#include <stm32f37x_conf.h >
#include "encoder.h"

//      
#define PhaseAB   ((GPIOA->IDR >> 6) & 0x03) //6,7

static volatile int32_t encoder_counter;

/*******************************************************************************
      UI
		Parameter:
		Return:
 Note:
*******************************************************************************/
void encoder_init (void)
{
    GPIO_InitTypeDef   GPIO_InitStructure;
    EXTI_InitTypeDef   EXTI_InitStructure;
    NVIC_InitTypeDef   NVIC_InitStructure;

    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

    SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource6);
    SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource7);

    EXTI_InitStructure.EXTI_Line = EXTI_Line6;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;  // falling-rising
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    EXTI_InitStructure.EXTI_Line = EXTI_Line7;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;  // falling-rising
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 5;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0; 			//    
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
}

/*******************************************************************************
          
		Parameter:
		Return:  
 Note:
 // http://easyelectronics.ru/repository.php?act=view&id=13
         4 
*******************************************************************************/

#define ENCODER_TRESHOLD    4

//     
const	int8_t	EncState[] = {0, -1, 1, 0, 1, 0, 0, -1, -1, 0, 0, 1, 0, 1, -1, 0};

static int32_t encoderHandler()
{
    static	int32_t	EncVal=0;
    static	uint8_t	Enc=0;
    static	int8_t	temp=0;

    Enc += PhaseAB ;			//     
    temp += EncState[Enc]; 		//      
    Enc <<= 2; 					//  4   
    Enc &= 0x0F;				//  4  

    if(temp > ENCODER_TRESHOLD )
    {
        temp = 0;
        EncVal++;
    }

    if(temp < -ENCODER_TRESHOLD )
    {
        temp = 0;
        EncVal--;
    }
    return (EncVal);
}

/*******************************************************************************
		Parameter:
		Return:  
*******************************************************************************/

int32_t get_encoder(void)
{
    return(encoder_counter);
}

/*******************************************************************************
*     PA ( )
 Note:
*******************************************************************************/
void EXTI9_5_IRQHandler(void)
{
    if(EXTI_GetITStatus(EXTI_Line6) != RESET)
    {
        encoder_counter = encoderHandler();
        EXTI_ClearITPendingBit(EXTI_Line6);
    }
    if(EXTI_GetITStatus(EXTI_Line7) != RESET)
    {
        encoder_counter = encoderHandler();
        EXTI_ClearITPendingBit(EXTI_Line7);
    }
}





